<?php

class dms{

	private $wettkampffolge = [
		[100,5], [200,1], [100,2], [200,3], [100,4],
		[50,2], [200,5], [1500,1], [50,4], [200,2],
		[100,3], [200,4], [50,1], [400,5], [50,3],
		[400,1], [100,1]
	];
	private $jugendschutz = [[1500,1], [200,4], [400,5]];
	private $db;
	private $wettkampfid = false;
	private $wettkampfliste = [];

	public function __construct($db, $wettkampfid = false){
		$this->db = $db;
		if($wettkampfid !== false and ctype_digit($wettkampfid)){
			$this->wettkampfid = $wettkampfid;
		}
		$this->baueWettkampfliste();
	}

	public function holeStreckenAnzahl(){
		return count($this->wettkampffolge)*2*2;
	}


	public function sucheWettkampfnummer($abschnitt, $disziplin, $strecke, $mw){
		foreach($this->wettkampfliste as $nr => $wk){
			if($wk['abschnitt'] != $abschnitt){
				continue;
			}
			if($wk['lage'] != $disziplin){
				continue;
			}
			if($wk['laenge'] != $strecke){
				continue;
			}
			if($wk['mw'] != $mw){
				continue;
			}
			return $nr;
		}
	}


	private function baueLeereZuordnung(){
		$strecken = [];
		foreach($this->wettkampffolge as $strecke){
			if(!isset($strecken[$strecke[1]])){
				$strecken[$strecke[1]] = [];
			}
			if(!isset($strecken[$strecke[1]][$strecke[0]])){
				$strecken[$strecke[1]][$strecke[0]] = false;
			}
		}
		foreach($strecken as $lage => $laengen){
			ksort($strecken[$lage]);
		}
		ksort($strecken);
		$array = [];
		foreach([1,2] as $abschnitt){
			$array[$abschnitt] = $strecken;
		}
		return $array;
	}


	public function leseStreckenzuordnung($_mannschaft = false){
		$_wkeinst = parse_ini_file(PFAD_INIDATEIEN.$_SESSION['wk_abwickeln']['dateiname'],true);
		$leer = $this->baueLeereZuordnung();
		$mannschaften = [];
		if(!isset($_wkeinst['dmsstreckenzuordnung'])){
			return $leer;
		}
		foreach($_wkeinst['dmsstreckenzuordnung'] as $mannschaft => $gespeichert){
			$strecken = $leer;
			$gespeichert = json_decode(str_replace("'",'"',$gespeichert), true);
			foreach($gespeichert as $abschnitt => $lagen){
				foreach($lagen as $lage => $laengen){
					foreach($laengen as $laenge => $mid){
						if($mid == '-'){
							continue;
						}
						$strecken[$abschnitt][$lage][$laenge] = $mid;
					}
				}
			}
			$mannschaften[$mannschaft] = $strecken;
		}
		if($_mannschaft !== false){
			if(isset($mannschaften[$_mannschaft])){
				return $mannschaften[$_mannschaft];
			}
			else{
				return false;
			}
		}
		return $mannschaften;
	}


	public function schreibeStreckenzuordnung($mannschaft, $formdaten){
		$strecken = $this->baueLeereZuordnung();
		$speichern = [];
		foreach($strecken as $abschnitt => $lagen){
			foreach($lagen as $lage => $laengen){
				foreach($laengen as $laenge => $mid){
					$form = $formdaten[$abschnitt][$lage][$laenge]??false;
					if($form === false or $form == '-'){
						continue;
					}
					if(!isset($speichern[$abschnitt])){
						$speichern[$abschnitt] = [];
					}
					if(!isset($speichern[$abschnitt][$lage])){
						$speichern[$abschnitt][$lage] = [];
					}
					if(!isset($speichern[$abschnitt][$lage][$laenge])){
						$speichern[$abschnitt][$lage][$laenge] = $form;
					}
				}
			}
		}
		# String bauen
		$streckenstr = false;
		if(count($speichern) > 0){
			$streckenstr = str_replace('"',"'",json_encode($speichern));
		}
		# Daten lesen und neu speichern
		$_wkeinst = parse_ini_file(PFAD_INIDATEIEN.$_SESSION['wk_abwickeln']['dateiname'],true);
		if(!isset($_wkeinst['dmsstreckenzuordnung'])){
			if($streckenstr === false){
				return;
			}
			$_wkeinst['dmsstreckenzuordnung'] = [];
		}
		if($streckenstr !== false){
			$_wkeinst['dmsstreckenzuordnung'][$mannschaft] = $streckenstr;
		}
		elseif(isset($_wkeinst['dmsstreckenzuordnung'][$mannschaft])){
			unset($_wkeinst['dmsstreckenzuordnung'][$mannschaft]);
			if(count($_wkeinst['dmsstreckenzuordnung'])==0){
				unset($_wkeinst['dmsstreckenzuordnung']);
			}
		}
		require_once(PFAD_INCLUDE.'func/schreibe_ini_datei.inc.php');
		schreibe_ini_datei(PFAD_INIDATEIEN.$_SESSION['wk_abwickeln']['dateiname'], $_wkeinst, true);
		return;
	}




	public function holeWettkampfliste($filter = [], $sort = false){
		$liste = $this->wettkampfliste;
		# Filtern (nach Geschlecht und/oder Abschnitt)
		if(array_key_exists('mw', $filter)){
			foreach($liste as $nr => $wk){
				if($filter['mw'] == $wk['mw']){
					continue;
				}
				unset($liste[$nr]);
			}
		}
		if(array_key_exists('abschnitt', $filter)){
			foreach($liste as $nr => $wk){
				if($filter['abschnitt'] == $wk['abschnitt']){
					continue;
				}
				unset($liste[$nr]);
			}
		}

		# ggf. sortieren
		if($sort == 'strecke'){
			usort($liste, function($a,$b){
				if($a['abschnitt'] == $b['abschnitt']){
					if($a['lage'] == $b['lage']){
						if($a['laenge'] == $b['laenge']){
							return $a['mw']<=>$b['mw'];
						}
						return $a['laenge']<=>$b['laenge'];
					}
					return $a['lage']<=>$b['lage'];
				}
				return $a['abschnitt']<=>$b['abschnitt'];
			});
		}
		return $liste;
	}


	private function baueWettkampfliste(){
		$wkgeschlecht = [];
		if($this->wettkampfid !== false){
			# das Geschlecht mit dem begonnen wird holen
			$query = "SELECT a.nr, w.mw
						FROM ".DB_PREFIX."abschnitte as a LEFT JOIN ".DB_PREFIX."wks as w ON a.id=w.abschnittid
						WHERE a.wettkampfid = ".$this->wettkampfid."
						ORDER BY a.nr, wknr DESC";
			$this->db->holeDaten($query);
			$wkgeschlecht = $this->db->baueArray('mw', 'nr');
		}

		# Wettkampffolge zusammenbauen
		$wettkaempfe = [];
		$i = 1;
		foreach([1,2] as $abschnitt){
			if(!isset($wkgeschlecht[$abschnitt])){
				$wkgeschlecht[$abschnitt] = 'w';
			}
			foreach($this->wettkampffolge as $strecke){
				$jugendschutz = in_array($strecke, $this->jugendschutz);
				$wk['w'] = [
					'wknr' => 0,
					'abschnitt' => $abschnitt,
					'laenge' => $strecke[0],
					'alaenge' => ($strecke[0]=='1500' and $abschnitt==1)?'800':$strecke[0],
					'lage' => $strecke[1],
					'mw' => 'w',
					'10nicht' => $jugendschutz
				];
				$wk['m'] = [
					'wknr' => 0,
					'abschnitt' => $abschnitt,
					'laenge' => $strecke[0],
					'alaenge' => ($strecke[0]=='1500' and $abschnitt==2)?'800':$strecke[0],
					'lage' => $strecke[1],
					'mw' => 'm',
					'10nicht' => $jugendschutz
				];
				$wk[$wkgeschlecht[$abschnitt]]['wknr'] = $i;
				$wettkaempfe[$i] = $wk[$wkgeschlecht[$abschnitt]];
				$i++;
				$wk[$wkgeschlecht[$abschnitt]=='w'?'m':'w']['wknr'] = $i;
				$wettkaempfe[$i] = $wk[$wkgeschlecht[$abschnitt]=='w'?'m':'w'];
				$i++;
			}
		}
		$this->wettkampfliste = $wettkaempfe;
	}




}
# EOF
